#include <iostream>
#include "dlgmain.h"
#include "globals.h"
#include "dlgstatus.h"
#include "dlgsharelist.h"
#include "syncprocess.h"
#include <qfiledialog.h>
#include <qbrush.h>
#include <qcolor.h>
#include <qpoint.h>

using namespace std;

DlgMain::DlgMain(SiteInfo &si, QWidget *parent, const char *name)
		: QWidget(parent, name) {
	siteInfo = &si;
	siteListBox = new QComboBox(true, this, "Site Name");
	hostEdit = new QLineEdit(this, "Host");
	remoteEdit = new QLineEdit(this, "Remote Path");
	localEdit = new QLineEdit(this, "Local Path");
	userEdit = new QLineEdit(this, "Username");
	passEdit = new QLineEdit(this, "Password");
	modeBox = new QButtonGroup(1, Qt::Vertical, this, "Mode container");
	modeDownBtn = new QRadioButton("D&ownload", modeBox, "Download");
	modeUpBtn = new QRadioButton("Upload", modeBox, "Upload");
	commentEdit = new QTextEdit(this, "Comment");
	portEdit = new QLineEdit(this, "Port");
	remoteSelBtn = new QPushButton("Loo&k up", this, "Remote Select");
	localSelBtn = new QPushButton("...", this, "Local Path Select");
	startBtn = new QPushButton("S&tart", this, "Start rsync");
	saveBtn = new QPushButton("&Save", this, "Save site");
	prevBtn = new QPushButton("&Prev", this, "Previous site");
	nextBtn = new QPushButton("&Next", this, "Next site");
	localhostBtn = new QCheckBox("&Local transfer", this, "Localhost");
	anonymousBtn = new QCheckBox("&Anonymous", this, "Anonymous");
	rootPermBtn = new QCheckBox("Use &root perms", this, "su root");
	widgetLayout = new QHBoxLayout(this, 0, -1, "Full widget layout");
	gridLeft = new QGridLayout(0, 15, 7, 0, -1, "Grid, left");
	gridRight = new QVBoxLayout(0, 0, -1, "Right side container");
	boxUserPass = new QHBoxLayout(0, 0, -1, "User/Pass container");
	statusDlg = new DlgStatus(0, "Status dialog");
	shareListDlg = new DlgShareList(0, "Share list dialog");
	syncProcess = new SyncProcess(*siteInfo, *statusDlg, *shareListDlg);
	statusDlg->hide();
	shareListDlg->hide();

	// have i mentioned i fucking hate labels?
	siteListLbl = new QLabel(siteListBox, "S&ite Name:", this, "Site Name");
	hostPortLbl = new QLabel(hostEdit, "&Host / Port:", this, "Host / Port");
	remoteLbl = new QLabel(remoteEdit, "Remot&e Path:", this, "Remote Path");
	localLbl = new QLabel(localEdit, "&Local Path:", this, "Local Path");
	userPassLbl = new QLabel(userEdit, "&User / Pass:", this, "Username / Pass");
	modeLbl = new QLabel("Transfer mode:", this, "Transfer mode");
	commentLbl = new QLabel(commentEdit, "&Comment:", this, "Comment");

	widgetLayout->addLayout(gridLeft);
	widgetLayout->addLayout(gridRight);
	gridLeft->setColStretch(MAIN_LBL_C, 1);
	gridLeft->addColSpacing(1, PADDING * 2);
	gridLeft->setColStretch(MAIN_FIELD_C, 50);
	gridLeft->addColSpacing(3, PADDING * 2);
	gridLeft->setColStretch(MAIN_AUX_C, 10);
	gridLeft->addColSpacing(5, PADDING * 10);
	gridLeft->setColStretch(MAIN_CTRL_C, 1);
	for (int i=0; i < 8; i++)
		gridLeft->addRowSpacing(i * 2, PADDING / 2);
	gridLeft->addWidget(siteListLbl, MAIN_SITE_R, MAIN_LBL_C);
	gridLeft->addWidget(siteListBox, MAIN_SITE_R, MAIN_FIELD_C);
	gridLeft->addWidget(hostPortLbl, MAIN_HOST_R, MAIN_LBL_C);
	gridLeft->addWidget(hostEdit, MAIN_HOST_R, MAIN_FIELD_C);
	gridLeft->addWidget(portEdit, MAIN_HOST_R, MAIN_AUX_C);
	gridLeft->addWidget(remoteLbl, MAIN_SRC_R, MAIN_LBL_C);
	gridLeft->addWidget(remoteEdit, MAIN_SRC_R, MAIN_FIELD_C);
	gridLeft->addWidget(remoteSelBtn, MAIN_SRC_R, MAIN_AUX_C);
	gridLeft->addWidget(localLbl, MAIN_DEST_R, MAIN_LBL_C);
	gridLeft->addWidget(localEdit, MAIN_DEST_R, MAIN_FIELD_C);
	gridLeft->addWidget(localSelBtn, MAIN_DEST_R, MAIN_AUX_C);
	boxUserPass->addWidget(userEdit);
	boxUserPass->addWidget(passEdit);
	gridLeft->addWidget(userPassLbl, MAIN_PASS_R, MAIN_LBL_C);
	gridLeft->addLayout(boxUserPass, MAIN_PASS_R, MAIN_FIELD_C);
	gridLeft->addWidget(modeLbl, MAIN_MODE_R, MAIN_LBL_C);
	gridLeft->addWidget(modeBox, MAIN_MODE_R, MAIN_FIELD_C);
	gridLeft->addWidget(commentLbl, MAIN_COMM_R, MAIN_LBL_C);
	gridLeft->addWidget(commentEdit, MAIN_COMM_R, MAIN_FIELD_C);
	gridRight->addWidget(startBtn);
	gridRight->addSpacing(PADDING * 4);
	gridRight->addWidget(saveBtn);
	gridRight->addSpacing(PADDING);
	gridRight->addWidget(prevBtn);
	gridRight->addSpacing(PADDING);
	gridRight->addWidget(nextBtn);
	gridRight->addSpacing(PADDING * 4);
	gridRight->addWidget(localhostBtn);
	gridRight->addWidget(anonymousBtn);
	gridRight->addWidget(rootPermBtn);
	gridRight->addStretch(50);

	connect(shareListDlg, SIGNAL(shareChosen(QString)), this, SLOT(remoteChosen(QString)));
	connect(remoteSelBtn, SIGNAL(clicked()), this, SLOT(getSource()));
	connect(localSelBtn, SIGNAL(clicked()), this, SLOT(getDestination()));
	connect(startBtn, SIGNAL(clicked()), this, SLOT(runSync()));
	connect(localhostBtn, SIGNAL(toggled(bool)), this, SLOT(setLocalhost(bool)));
	connect(anonymousBtn, SIGNAL(toggled(bool)), this, SLOT(setAnonymous(bool)));
	connect(hostEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changeHost(const QString &)));
	connect(portEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changePort(const QString &)));
	connect(passEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changePass(const QString &)));
	connect(userEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changeUser(const QString &)));
	connect(remoteEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changeRemote(const QString &)));
	connect(localEdit, SIGNAL(textChanged(const QString &)), this, SLOT(changeLocal(const QString &)));
	//connect(modeDownBtn, SIGNAL(toggled(bool)), this, SLOT(modeDownTgl(bool)));
	//connect(modeUpBtn, SIGNAL(toggled(bool)), this, SLOT(modeUpTgl(bool)));

	minSizeSet = false;
	passEdit->setEchoMode(QLineEdit::Password);
	portEdit->setMaxLength(PORT_LENGTH);
	portEdit->setText(siteInfo->getPort().c_str());
	modeBox->setLineWidth(0);
	modeBox->setInsideMargin(0);
	if (!modeBox->selected())
		modeDownBtn->toggle();
	rootPermBtn->setDisabled(true);	

	statusDlg->resize(800, 400);

	modeDownBtn->setDisabled(true);
	modeUpBtn->setDisabled(true);
	remoteSelBtn->setDisabled(true);
	localhostBtn->setDisabled(true);
}

void DlgMain::paintEvent(QPaintEvent *event) {
	QPainter p(this);
	event = event; // get rid of unused var warning

	p.drawLine(startBtn->frameGeometry().left() + PADDING, startBtn->frameGeometry().bottom() + (PADDING * 2), startBtn->frameGeometry().right() - PADDING, startBtn->frameGeometry().bottom() + (PADDING * 2));
	p.drawLine(nextBtn->frameGeometry().left() + PADDING, nextBtn->frameGeometry().bottom() + (PADDING * 2), nextBtn->frameGeometry().right() - PADDING, nextBtn->frameGeometry().bottom() + (PADDING * 2));

	// resize the QTabWidget to closely bound this dialog's contents
	// FIXME: I'd like to dynamically figure out the QTabBar's height instead of using 30...
	if (!minSizeSet) {
		parentWidget()->parentWidget()->setMinimumSize(500, 250);
		parentWidget()->parentWidget()->resize(500, 250);
		minSizeSet = true;
	}
}

void DlgMain::updateInfo(SiteInfo &si) {
	siteInfo = &si;
	hostEdit->setText(siteInfo->getHost().c_str());
	remoteEdit->setText(siteInfo->getRemote().c_str());
	localEdit->setText(siteInfo->getLocalDir().c_str());
	userEdit->setText(siteInfo->getUser().c_str());
	passEdit->setText(siteInfo->getPass().c_str());
}

void DlgMain::changeLocal(const QString &s) {
	siteInfo->setLocalDir((const char *)s);
}

void DlgMain::changeHost(const QString &s) {
	siteInfo->setHost((const char *)s);
}

void DlgMain::changePort(const QString &s) {
	siteInfo->setPort((const char *)s);
}

void DlgMain::changeUser(const QString &s) {
	siteInfo->setUser((const char *)s);
}

void DlgMain::changePass(const QString &s) {
	siteInfo->setPass((const char *)s);
}

void DlgMain::changeRemote(const QString &s) {
	siteInfo->setRemote((const char *)s);
	syncProcess->setNoShareSpecified((s ? false : true));
}

void DlgMain::remoteChosen(const QString s) {
	remoteEdit->setText(s); // changeRemote() is called via signal
	runSync();
}

// TODO: eventually allow file selection after verifying that source is file, not dir.
void DlgMain::getDestination() {
	QFileDialog destDlg;
	QString dir;
	if ((dir = destDlg.getExistingDirectory(NULL, this, "dest selection dlg", "Select Destination Directory")))
		localEdit->setText(dir);
}

// TOFIX: SOURCE NEEDS DIR SEL TOO, NOT ONLY FILE!
// TOFIX: BOTH Source/Dest CANNOT BE FILLED WIHTOUT LOCALHOST... RIGHT?
	
// TODO: eventually allow multiple file selection... needs implementation elsewhere too.
void DlgMain::getSource() {
	QFileDialog sourceDlg;
	QString dir;
	if ((dir = sourceDlg.getExistingDirectory(NULL, this, "src selection dlg", "Select Source Directory")))
		remoteEdit->setText(dir);
}

void DlgMain::setAnonymous(bool checked) {
	if (checked) {
		userEdit->setDisabled(true);
		passEdit->setDisabled(true);
	}
	else {
		userEdit->setEnabled(true);
		passEdit->setEnabled(true);
	}
}

void DlgMain::setLocalhost(bool checked) {
	if (checked) {
		hostEdit->setText(LOCALHOST_T);
		hostEdit->setDisabled(true);
		portEdit->setText(siteInfo->getPort().c_str());
		portEdit->setDisabled(true);
		remoteSelBtn->setEnabled(true);
		localSelBtn->setEnabled(true);
		modeBox->setDisabled(true);
		siteInfo->setLocalhost(true);
	}
	else {
		hostEdit->setEnabled(true);
		hostEdit->setText(siteInfo->getHost().c_str());
		portEdit->setEnabled(true);
		portEdit->setText(siteInfo->getPort().c_str());
		modeBox->setEnabled(true);
		/*
		// UGLY.  GAY.  needed to ensure correct "..." button's enabled.  BLAH.
		if (modeBox->selected() == modeDownBtn) {
			modeUpBtn->toggle();
			modeDownBtn->toggle();
		}
		else {
			modeDownBtn->toggle();
			modeUpBtn->toggle();
		}
		*/
		siteInfo->setLocalhost(false);
	}
}

void DlgMain::runSync() {
	syncProcess->go(RSyncStuff::ChangeDir);
}
