#ifndef SYNCPROCESS_H
#define SYNCPROCESS_H

#include "globals.h"
#include "siteinfo.h"
#include "dlgstatus.h"
#include "dlgsharelist.h"
#include <qprocess.h>
#include <qvaluelist.h>
#include <qfile.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qdatetime.h>

class SyncProcess : public QObject {
		Q_OBJECT
	public:
		SyncProcess(const SiteInfo &si, DlgStatus &ds, DlgShareList &dsl);
		void go(RSyncStuff::RSyncOperation op);
	public slots:
		void readOutput();
		void processOutput();
		void readError();
		void viewDir(QString s);
		void getFile(QString s);
		void sendFile(QString s);
		void setNoShareSpecified(bool b);
	signals:
		void percentChanged(int);
		void shareListShown();
	private slots:
		void postSync();
	private:
		void printStatus(QString s, bool isError=false, bool addEndl=true);
		void processMOTDSpew();
		void displayMOTD();
		// TODO: unnecessary other than testing? void displayShares();
		enum ArgType { Command, Option, Source, Destination, Remote, Local };
		void resetState();
		void runProcess();
		void syncOperation(RSyncStuff::RSyncOperation op, QString s);
		void writePassFile();
		QProcess *rsync;
		SiteInfo *siteInfo;
		DlgStatus *statusDialog;
		DlgShareList *shareListDialog;
		bool readingOutput, readingError, readyOutput, readyError,
			connectDone,
			genRunning, genFilesP1, genFilesP2, recvDone, fileMatched,
			ranOnce, modeIsDownload, loadingDir, noShareSpecified,
			syncRunning;
		// new vars... above ones may go away
		bool recvFileInfo, motdNext, motdReceived, recvListStart, recvListDone,
			xferringFiles, listingOnly;
		int fileCountList, fileRemovedList, filesRemain, generatorPID, motdCounter;
		QDateTime syncStartTime, syncEndTime, fileStartTime, fileEndTime;
		QFile passFile;
		QString currentFile;
		QStringList filenames, motd, output; //, shareList;
		QStringList::iterator itFilenames;
		QValueList<ArgType> argTypeList; 
};

#endif /* SYNCPROCESS_H */
